//There should be no reason to edit this file at all. Just understand that it
//is part of a system to make defining new server commands, client commands, and cvars
//easier to do.

#ifdef DEFINE_CLIENT_COMMAND
#undef DEFINE_CLIENT_COMMAND
#endif
#define DEFINE_CLIENT_COMMAND(function) //by default this macro should do nothing at all

#ifdef DEFINE_SERVER_COMMAND
#undef DEFINE_SERVER_COMMAND
#endif
#define DEFINE_SERVER_COMMAND(function) //by default this macro should do nothing at all

#ifdef DEFINE_CVAR
#undef DEFINE_CVAR
#endif
#define DEFINE_CVAR(cvarname, cvarflags, cvarstringvalue, cvarfloatvalue) //by default this macro should do nothing at all


//DEFINE_CLIENT_COMMAND() Macro
//This macro can be defined in two different ways depending on whether the
//REGISTERING_CLIENT_COMMANDS macro is defined, if that macro is undefined 
//then DEFINE_CLIENT_COMMAND() will simply provide a forward declaration of the 
//function in the form void function(CBasePlayer *pPlayer), if the 
//REGISTERING_CLIENT_COMMANDS macro is defined then DEFINE_CLIENT_COMMAND() 
//will run the necessary code to detect if a client has typed the specified
//command and call the associated function
//
//Why you ask: This complex macro exists so that I can add a function to this
//one file and have it all setup and ready to go in every place that needs
//to do something to prepare it. In short: to centralize information

//START - DEFINE_CLIENT_COMMAND() Macro
#ifdef REGISTERING_CLIENT_COMMANDS
#undef DEFINE_CLIENT_COMMAND
#define DEFINE_CLIENT_COMMAND(function) \
	if(FStrEq(szUserCommand, "" #function "")) { \
		function( (CBasePlayer *)GET_PRIVATE(pEntity) ); \
		return; \
	} else
#endif // #ifdef REGISTERING_CLIENT_COMMANDS

#ifdef CLIENT_COMMAND_DEFINITIONS
#undef DEFINE_CLIENT_COMMAND
#define DEFINE_CLIENT_COMMAND(function) void function(CBasePlayer *pPlayer)
#endif //#ifdef CLIENT_COMMAND_DEFINITIONS
//END - DEFINE_CLIENT_COMMAND() Macro

//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================

//DEFINE_SERVER_COMMAND() Macro
//This macro can be defined in two different ways depending on whether the
//REGISTERING_SERVER_COMMANDS macro is defined, if that macro is undefined then
//DEFINE_SERVER_COMMAND() will simply provide a forward declaration of the function
//in the form void function(void), if the REGISTERING_SERVER_COMMANDS macro is defined
//then DEFINE_SERVER_COMMAND() will run REG_SVR_COMMAND("function", function)
//
//Why you ask: This complex macro exists so that I can add a function to this
//one file and have it all setup and ready to go in every place that needs
//to do something to prepare it. In short: to centralize information

//START - DEFINE_SERVER_COMMAND() Macro
#ifdef REGISTERING_SERVER_COMMANDS
#undef DEFINE_SERVER_COMMAND
#define DEFINE_SERVER_COMMAND(function) REG_SVR_COMMAND( "" #function "", function)
#endif //#ifdef REGISTERING_SERVER_COMMANDS

#ifdef SERVER_COMMAND_DEFINITIONS
#undef DEFINE_SERVER_COMMAND
#define DEFINE_SERVER_COMMAND(function) void function(void)
#endif
//END - DEFINE_SERVER_COMMAND() Macro

//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================

//DEFINE_CVAR() Macro
//This Macro is used to take care of all registration and initialization of serverside
//CVARS. The macro does 3 different things depending on where this file is included
//1.	If included after REGISTERING_CVARS is defined, then it does CVAR_REGISTER against
//		a global registration cvar, and then sets a global cvar_t * with the name of
//		"cvar_<thesuppliednamehere>" to point to the engine's copy of the cvar
//2.	If included after DO_CVAR_EXTERNS is defined, then this creates extern definitions
//		of all cvar_t * variables that the macro will initialize under case #1
//3.	If included after CREATE_CVAR_VARIABLES is defined, then this creates the actual
//		instances of the cvar_t * variables mentioned in case #1 as well as a one-time
//		cvar_t variable with the requested values for registering the cvar


//START - DEFINE_CVAR() Macro
#ifdef REGISTERING_CVARS
#undef DEFINE_CVAR
#define DEFINE_CVAR(cvarname, cvarflags, cvarstringvalue, cvarfloatvalue) \
CVAR_REGISTER(&cvarname ## _register ); cvar_ ## cvarname = CVAR_GET_POINTER( "" #cvarname "" );
#endif //#ifdef REGISTERING_CVARS

#ifdef CREATE_CVAR_EXTERNS
#undef DEFINE_CVAR
#define DEFINE_CVAR(cvarname, cvarflags, cvarstringvalue, cvarfloatvalue)\
		extern cvar_t * cvar_ ## cvarname; 
#endif //#ifdef DO_CVAR_EXTERNS

#ifdef CREATE_CVAR_VARIABLES
#undef DEFINE_CVAR
#define DEFINE_CVAR(cvarname, cvarflags, cvarstringvalue, cvarfloatvalue)\
	cvar_t cvarname ## _register = { "" #cvarname "", cvarstringvalue, cvarflags, cvarfloatvalue };\
	cvar_t * cvar_ ## cvarname;
#endif //#ifdef CREATE_CVAR_VARIABLES
//END - DEFINE_CVAR() Macro

//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================
//========================================================================================

#define SETTING_UP_CVARS_AND_COMMANDS
#include "plugin.h"
#undef SETTING_UP_CVARS_AND_COMMANDS



