#include "plugin.h"
#include "plugin_cbase.h"

#define CLIENT_COMMAND_DEFINITIONS
#include "complexmacros.h"
#undef CLIENT_COMMAND_DEFINITIONS

#include <sdk_util.h>
#include <usercmd.h>

void ClientCommand(edict_t *pEntity)
{
	const char* szUserCommand = CMD_ARGV(0);

	/* to get rid of warnings if the plugin doesn't make any client commands */
	if (szUserCommand[0] == '\0') 
		return;

	int PlIndex = ENTINDEX(pEntity);
	CBaseEntity *pPlayer = UTIL_PlayerByIndex(PlIndex);

#if PG_DEBUG_MODE
	char sTemp[128] = "";
#endif

	if (DynamicSizedData.PlayerData[PlIndex].iInMenu)
	{
		if (FStrEq(CMD_ARGV(0), "menuselect"))
		{
			UTIL_DisableMenu(pPlayer);

			phasegate_t *pPG = DynamicSizedData.PlayerData[PlIndex].pLastEntUsed;

			if (pPG == NULL)
				return;

			int iOption = atoi(CMD_ARGV(1));
			int PgIndex = pPG->iEntityIndex;
			
			/* Now we determine which menu to use */
			if ((int)cvar_phaseselect_pg->value == 3)
			{
				int iThisNum = 1;

				/* We use the PhaseSelect menu */
				if (iOption != 8 && iOption != 9)
				{
					/* If the player hit the next menu button, make sure they get to that pg,
					   not the one in the first menu! */
					if (DynamicSizedData.PlayerData[PlIndex].iInMenu > 1)
						iOption += (DynamicSizedData.PlayerData[PlIndex].iInMenu - 1);

#if PG_DEBUG_MODE
					sprintf ( sTemp, "Menuselect: %d. Starting from: %d.\n", iOption, DynamicSizedData.PlayerData[PlIndex].iInMenu );
					ALERT( at_aiconsole, sTemp );
#endif

					int iIndices = 1;

					phasegate_t	*pPhaseGate = DynamicSizedData.pPhaseGateHead;

					while (pPhaseGate != NULL)
					{
						if (pPhaseGate->iEntityIndex == PgIndex)
						{
							pPhaseGate = pPhaseGate->pNext;
							continue;
						}
#if PG_DEBUG_MODE
						sprintf( sTemp, "Phasegate %d (Need %d). Entid: %d. Vector( x=%f, y=%f, z=%f ).\n", iIndices, iOption, pPhaseGate->iEntityIndex, pPhaseGate->vecOrigin.x, pPhaseGate->vecOrigin.y, pPhaseGate->vecOrigin.z );
						ALERT( at_aiconsole, sTemp );
#endif
						if (iIndices == iOption)
							break;

						iIndices++;

						pPhaseGate = pPhaseGate->pNext;
					}

					if (iIndices == iOption && pPhaseGate != NULL)
					{
#if PG_DEBUG_MODE
						ALERT( at_aiconsole, "PhaseGate found\n" );
#endif
						pPG = pPhaseGate;

					}
					else
					{
#if PG_DEBUG_MODE
						ALERT( at_aiconsole, "PhaseGate not found\n" );
#endif

						DynamicSizedData.PlayerData[PlIndex].iInMenu = 0;
						DynamicSizedData.PlayerData[PlIndex].flMenuDelay = 0.0f;
						DynamicSizedData.PlayerData[PlIndex].pLastEntUsed = NULL;

						RETURN_META(MRES_SUPERCEDE);
					}
				} /* iOption */
				else if (iOption == 9)
				{
#if PG_DEBUG_MODE
					ALERT( at_aiconsole, "Menu Exit\n" );
#endif
					DynamicSizedData.PlayerData[PlIndex].iInMenu = 0;
					DynamicSizedData.PlayerData[PlIndex].flMenuDelay = 0.0f;
					DynamicSizedData.PlayerData[PlIndex].pLastEntUsed = NULL;

					RETURN_META(MRES_SUPERCEDE);
				} /* iOption */
				else
				{
#if PG_DEBUG_MODE
					ALERT( at_aiconsole, "Menu Expand\n" );
#endif

					char szMessage[512] = "Select Exit:\n\n";
					char szTemp[64] = "";

					phasegate_t	*pPhaseGate = DynamicSizedData.pPhaseGateHead;

					/* The iInPhaseMenu value determines which PhaseGate beyond */
					iThisNum = (DynamicSizedData.PlayerData[PlIndex].iInMenu == 1) ? 8 : DynamicSizedData.PlayerData[PlIndex].iInMenu;

					while (pPhaseGate != NULL)
					{
						if (pPhaseGate->iEntityIndex == PgIndex)
						{
							pPhaseGate = pPhaseGate->pNext;
							continue;
						}

						if (--iThisNum == 0)
							break;

#if PG_DEBUG_MODE
						sprintf( sTemp, "Phasegate %d (Need 0). Entid: %d. Vector( x=%f, y=%f, z=%f ).\n", iOption, pPhaseGate->iEntityIndex, pPhaseGate->vecOrigin.x, pPhaseGate->vecOrigin.y, pPhaseGate->vecOrigin.z );
						ALERT( at_aiconsole, sTemp );
#endif

						pPhaseGate = pPhaseGate->pNext;
					}

					iThisNum = 1;

					/* Create the text */
					while (pPhaseGate != NULL)
					{
						edict_t *pEdict = INDEXENT(pPhaseGate->iEntityIndex);

						/* Only do 8 at a time. */
						if (iThisNum == 8)
							break;

						/* Skip same phasegate */
						if (pPhaseGate->iEntityIndex != PgIndex)
						{
							sprintf( szTemp, "%d. %s\n", iThisNum, UTIL_GetLocation(pPhaseGate->vecOrigin) );
							strcat( szMessage, szTemp );
						}
						else
						{
							pPhaseGate = pPhaseGate->pNext;
							continue;
						}

						/* Make sure we don't create an access violation */
						if (pPhaseGate != NULL)
						{
							iThisNum++;
							pPhaseGate = pPhaseGate->pNext;
						}
					}

					unsigned short uSlots = (1<<0) | (1<<8);

					if (iThisNum == 8 && (((DynamicSizedData.iNumPhases - 1) % 7) != 0))
					{
						uSlots |= (1<<7);
						strcat( szMessage, "8. More\n" );
					}

					strcat( szMessage, "\n9. Exit\n" );

					DynamicSizedData.PlayerData[PlIndex].iInMenu += 7;
					DynamicSizedData.PlayerData[PlIndex].pLastEntUsed = pPG;

					for (int i = 1; i < (iThisNum - 1); i++)
					{
						uSlots |= (1<<i);
					}

					/* Them's uber menu, combined together */
					UTIL_EnableMenu( pPlayer, szMessage, uSlots );

					RETURN_META(MRES_SUPERCEDE);
				} /* iOption */

				DynamicSizedData.PlayerData[PlIndex].iInMenu = 0;
				DynamicSizedData.PlayerData[PlIndex].flMenuDelay = gpGlobals->time + cvar_phaseselect_delay->value;
				DynamicSizedData.PlayerData[PlIndex].pLastEntUsed = NULL;

				UTIL_MovePlayer( pEntity, pPG->vecOrigin );

			} /* cvar_phaseselect_pg->value */
			else if ((int)cvar_phaseselect_pg->value == 2)
			{
				if (iOption == 1)
				{
					if (pPG->pNext == NULL)
						pPG = DynamicSizedData.pPhaseGateHead;
					else
						pPG = pPG->pNext;
				}
				else if (iOption == 2)
				{
					if (pPG->pPrev == NULL)
						pPG = DynamicSizedData.pPhaseGateTail;
					else
						pPG = pPG->pPrev;
				}
				else
				{
					DynamicSizedData.PlayerData[PlIndex].iInMenu = 0;
					DynamicSizedData.PlayerData[PlIndex].flMenuDelay = 0.0f;
					DynamicSizedData.PlayerData[PlIndex].pLastEntUsed = NULL;
					RETURN_META(MRES_SUPERCEDE);
				}

				DynamicSizedData.PlayerData[PlIndex].iInMenu = 0;

				UTIL_MovePlayer( pEntity, pPG->vecOrigin );
			} /* cvar_phaseselect_pg->value */

			RETURN_META(MRES_SUPERCEDE);
		}	
	}

	//these next 3 lines will use the super duper DEFINE_CLIENT_COMMAND() macro to route each command to the appropriate function
	#define REGISTERING_CLIENT_COMMANDS
		#include "complexmacros.h"
	#undef REGISTERING_CLIENT_COMMANDS
	
	RETURN_META(MRES_IGNORED);
}

void CmdStart( const edict_t *player, const struct usercmd_s *_cmd, unsigned int random_seed )
{
	edict_t *pEntity = (edict_t *)player;

	int PlIndex = ENTINDEX(pEntity);
	struct usercmd_s * cmd = (struct usercmd_s *)_cmd;

	if (PlIndex < 1 || PlIndex > 32)
		RETURN_META(MRES_IGNORED);

	/* If they drop a phasegate, and the limitpg cvar is in effect */
	if (cmd->impulse == 55 && (int)cvar_phaseselect_limitpg->value != 0)
	{
		/* Count the number of active phasegates */
		if (UTIL_CountBuildingsTeam("phasegate", pEntity->v.team) >= (int)cvar_phaseselect_limitpg->value)
		{
			char szString[48] = "";
			sprintf( szString, "* Max PhaseGates Reached (%d PhaseGates)\n", (int)cvar_phaseselect_limitpg->value );

			if (DynamicSizedData.gmsgTextMsg == 0)
				SAFE_USER_MSG(DynamicSizedData.gmsgTextMsg, "TextMsg", -1 );
			
			/* Tell them they can't build anymore */
			MESSAGE_BEGIN( MSG_ONE, DynamicSizedData.gmsgTextMsg, NULL, pEntity );
			WRITE_BYTE( HUD_PRINTTALK );
			WRITE_STRING( szString );
			MESSAGE_END();

			cmd->impulse = 0;
			RETURN_META(MRES_SUPERCEDE);
		}
	}

	/* If they are in the PhaseMenu */
	if (DynamicSizedData.PlayerData[PlIndex].iInMenu)
	{
		CBaseEntity *pPlayer = UTIL_PlayerByIndex(PlIndex);

		/* If they try to move in any direction, cancel the menu */
		if	  (cmd->buttons & IN_FORWARD 
			|| cmd->buttons & IN_BACK
			|| cmd->buttons & IN_MOVELEFT
			|| cmd->buttons & IN_MOVERIGHT)
		{
			DynamicSizedData.PlayerData[PlIndex].iInMenu = 0;
			DynamicSizedData.PlayerData[PlIndex].pLastEntUsed = NULL;
			UTIL_DisableMenu(pPlayer);
		}
	}
	RETURN_META(MRES_HANDLED);
}

/* For setthink and also remapping the nodes in the linked list */
void StartFrame( void )
{
	/* These are done only every 250ms */
	if (DynamicSizedData.NextEntityCheck < gpGlobals->time)
	{
		/* Update vars */
		TextMenuSettings.r1 = (unsigned char)cvar_phaseselect_menu_red->value;
		TextMenuSettings.g1 = (unsigned char)cvar_phaseselect_menu_green->value;
		TextMenuSettings.b1 = (unsigned char)cvar_phaseselect_menu_blue->value;
		TextMenuSettings.x = cvar_phaseselect_menu_x->value;
		TextMenuSettings.y = cvar_phaseselect_menu_y->value;
		TextMenuSettings.channel = (int)cvar_phaseselect_menu_channel->value;
		
		/* Fix the delay so its only positive (multiply by -1 if less than 0)
		   Otherwise leave it alone (multiply it by +1) */
		cvar_phaseselect_delay->value *= ((int)cvar_phaseselect_delay->value < 0) ? -1 : 1;

		CBaseEntity *pCPhaseGate = NULL;

		/* Phasegates dont normally think, so we have to make them (think) */
		while (pCPhaseGate = UTIL_FindEntityByClassname(pCPhaseGate, "phasegate"))
		{
			/* This is used to set the think */
			if (pCPhaseGate->pev->iuser1 == 0)
			{
				pCPhaseGate->pev->nextthink = gpGlobals->time + 0.1;
				pCPhaseGate->pev->iuser1 = 1;
			}
		}

		DynamicSizedData.NextEntityCheck = gpGlobals->time + 0.25;
	}

#if PG_DEBUG_MODE
	char sTemp[512] = "";
#endif

	phasegate_t *pPhaseGate = DynamicSizedData.pPhaseGateHead;

	/* Remap if a pg was destroyed. */
	while (pPhaseGate != NULL)
	{
		if (DynamicSizedData.pPhaseGateHead == NULL)
			break;

		edict_t *pEdict = INDEXENT(pPhaseGate->iEntityIndex);

		/* It's null/dead */
		if (FNullEnt(pEdict))
		{
			phasegate_t *pNextPhaseGate = NULL;

#if PG_DEBUG_MODE
			sprintf( sTemp, "Phasegate ID #%d destroyed at x=%f y=%f z=%f.\n", pPhaseGate->iEntityIndex, pPhaseGate->vecOrigin.x, pPhaseGate->vecOrigin.y, pPhaseGate->vecOrigin.z );
			ALERT( at_aiconsole, sTemp );
#endif

			pNextPhaseGate = pPhaseGate->pNext;
			DynamicSizedData.RemoveNode(pPhaseGate);
			pPhaseGate = pNextPhaseGate;

			continue;
		}
		
		if (pPhaseGate != NULL)
			pPhaseGate = pPhaseGate->pNext;
	}

	RETURN_META(MRES_HANDLED);
}

int DispatchSpawn( edict_t *pEntity ) 
{
	if (FStrEq(STRING(pEntity->v.classname), "worldspawn"))
	{
		char mapname[255] = "";
		strcpy(mapname, STRING(gpGlobals->mapname));

		if (mapname[0]=='c' && mapname[1]=='o' && mapname[2]=='_')
			DynamicSizedData.iCombat = 1;

		DynamicSizedData.Initialize();

		RETURN_META_VALUE(MRES_HANDLED, 0);
	}

	RETURN_META_VALUE(MRES_IGNORED, 0);
}

void DispatchThink( edict_t *pent )
{
	if (FNullEnt(pent))
		RETURN_META(MRES_IGNORED);

#if PG_DEBUG_MODE
	char sTemp[512] = "";
#endif

	if (FStrEq(STRING(pent->v.classname), "phasegate"))
	{
		/* The PG hasn't been fully built yet */
		if (pent->v.iuser1 == 1)
		{
			if (UTIL_IsBuilt(pent))
			{
				pent->v.iuser1 = 2;

				phasegate_t *pPhaseGate = new phasegate_t;

				if (pPhaseGate != NULL)
				{
					pPhaseGate->iEntityIndex = ENTINDEX(pent);
					pPhaseGate->vecOrigin = pent->v.origin;
					pPhaseGate->pNext = NULL;
					pPhaseGate->pPrev = NULL;

					DynamicSizedData.AppendNode(pPhaseGate);
#if PG_DEBUG_MODE
					sprintf( sTemp, "Phasegate completed as %d, at x=%f y=%f z=%f.\n", pPhaseGate->iEntityIndex, pPhaseGate->vecOrigin.x, pPhaseGate->vecOrigin.y, pPhaseGate->vecOrigin.z );
					ALERT( at_aiconsole, sTemp );
#endif
				}
			}
		}
		/* The PG is able to be used */
		else if (pent->v.iuser1 == 2)
		{
			/* Fix for custom maps with phasegates */
			if (UTIL_IsBuilt(pent) && !DynamicSizedData.iCombat)
			{
				CBaseEntity *pEnt = CBaseEntity :: Instance(pent);
				
				/* No uglies */
				if (FNullEnt(pEnt->pev))
					RETURN_META(MRES_IGNORED);

				int iPGVar = (int)cvar_phaseselect_pg->value;

				if (pEnt->m_pfnUse != &CBaseEntity :: m_pfnPhaseUse && iPGVar != 0)
					pEnt->m_pfnUse =  &CBaseEntity :: m_pfnPhaseUse;
			}
		}
		/* Fix if the iuser1 gets out of hand */
		else
		{
			pent->v.iuser1 = 1;
		}

		pent->v.nextthink = gpGlobals->time + 0.25;
	}

	RETURN_META(MRES_HANDLED);
}

BOOL ClientConnect( edict_t *pEntity, const char *pszName, const char *pszAddress, char szRejectReason[ 128 ] )
{
	memset(&DynamicSizedData.PlayerData[ENTINDEX(pEntity)], 0, sizeof(playerdata_t));
	RETURN_META_VALUE(MRES_HANDLED,TRUE);
}

void PlayerPreThink( edict_t *pEntity )
{
	int PlIndex = ENTINDEX(pEntity);
	
	/* Don't let the players phase into a phasegate that doesn't exist */
	if (DynamicSizedData.PlayerData[PlIndex].iInMenu)
	{
		linkedentity_s *pEnt = DynamicSizedData.PlayerData[PlIndex].pLastEntUsed;
		CBaseEntity *pPlayer = UTIL_PlayerByIndex(PlIndex);
		int PgIndex = 0;

		if (pEnt != NULL)
			PgIndex = pEnt->iEntityIndex;

		if (FNullEnt(PgIndex))
		{
			DynamicSizedData.PlayerData[PlIndex].iInMenu = 0;
			UTIL_DisableMenu( pPlayer );
#if PG_DEBUG_MODE
			ALERT( at_aiconsole, "NullEnt found on PlayerPreThink. Disabled Menu.\n" );
#endif
		}
	}

	RETURN_META(MRES_HANDLED);
}

/* New method, why the hell didnt I do this before. wheee! */
void DispatchKeyValue( edict_t *pentKeyvalue, KeyValueData *pkvd )
{
	if (FClassnameIs(pentKeyvalue, "info_location") && FStrEq(pkvd->szKeyName, "locationname") )
	{
		char szName[128];
		strcpy(szName, pkvd->szValue);
		UTIL_LookupLocationTitle(szName);

		/* This is needed in order for the mins and maxs to be other than 0 */
		SET_MODEL(pentKeyvalue, STRING(pentKeyvalue->v.model));
		SET_SIZE(pentKeyvalue, pentKeyvalue->v.mins, pentKeyvalue->v.maxs);

		strcpy(DynamicSizedData.LocationInfo[++DynamicSizedData.NumLocations].Name, szName);

		DynamicSizedData.LocationInfo[DynamicSizedData.NumLocations].Origin[0] = pentKeyvalue->v.mins;
		DynamicSizedData.LocationInfo[DynamicSizedData.NumLocations].Origin[1] = pentKeyvalue->v.maxs;

		RETURN_META(MRES_HANDLED);
	}
	RETURN_META(MRES_IGNORED);
}

void MessageBegin( int msg_dest, int msg_type, const float *pOrigin, edict_t *ed )
{
	/* This is the only place I could figure out where to put this. */
	if (!DynamicSizedData.bDidMsg)
	{
		SAFE_USER_MSG(DynamicSizedData.gmsgTextMsg,		"TextMsg", -1);
		SAFE_USER_MSG(DynamicSizedData.gmsgScreenFade,	"ScreenFade", 12);
		SAFE_USER_MSG(DynamicSizedData.gmsgShowMenu,	"ShowMenu", -1);
		DynamicSizedData.bDidMsg = true;
	}

	RETURN_META(MRES_HANDLED);
}
