//This should contain plugin information and declarations we need throughout the entire plugin

//NOTE:	I know this has nothing to do with plugin information, but this seems like the one place
//		every person is likely to read......
//		You need to make sure that you edit your Makefile or your MSVC project settings so that
//		the proper "include" directories get used. In MSVC you can change your include directories
//		by going to Project->Settings, change the "Settings For" to "All Configurations", click
//		the C/C++ tab, change the category to "Preprocessor", then use "Additional include directories"

//		You'll need to include the metamod source code directory, then the following from the 
//		multiplayer directory of the Half-Life SDK: common,engine,dlls,pm_shared

#ifdef SETTING_UP_CVARS_AND_COMMANDS

//USE THIS AREA TO SETUP:
//1. CVARS
//2. Serverside Commands
//3. Client Commands

//CVARS
//syntax: DEFINE_CVAR(cvar name, cvar flags, cvar default string value, cvar default float value );
//EXAMPLE: DEFINE_CVAR(mp_friendlyfire, FCVAR_SERVER | FCVAR_SPONLY | FCVAR_PRINTABLEONLY , "0", 0 );
//	the cvar would then be accessible as a cvar_t* named cvar_mp_friendlyfire

// Delay between uses of phasegates and movement chambers
DEFINE_CVAR( phaseselect_delay, FCVAR_SERVER, "1.5", 1.5f );

// Menu cvars
DEFINE_CVAR( phaseselect_menu_channel, FCVAR_SERVER, "28", 28 );
DEFINE_CVAR( phaseselect_menu_x, FCVAR_SERVER, "0.6", 0.6f );
DEFINE_CVAR( phaseselect_menu_y, FCVAR_SERVER, "0.35", 0.35f );
DEFINE_CVAR( phaseselect_menu_red, FCVAR_SERVER, "255", 255 );
DEFINE_CVAR( phaseselect_menu_green, FCVAR_SERVER, "255", 0 );
DEFINE_CVAR( phaseselect_menu_blue, FCVAR_SERVER, "255", 0 );

DEFINE_CVAR( phaseselect_pg_use, FCVAR_SERVER, "0", 0.0f );

DEFINE_CVAR( phaseselect_pg, FCVAR_SERVER, "3", 3.0f );
DEFINE_CVAR( phaseselect_limitpg, FCVAR_SERVER, "0", 0.0f );
//DEFINE_CVAR( phaseselect_mc, FCVAR_SERVER, "0", 0.0f );
//DEFINE_CVAR( phaseselect_limitmc, FCVAR_SERVER, "0", 0.0f );

//SERVER COMMANDS
//DEFINE_SERVER_COMMAND( exampleServerCommand );
//	be sure to create the function body using the example in servercommands.cpp



//CLIENT COMMANDS
//DEFINE_CLIENT_COMMAND( exampleClientCommand );
//	be sure to create the function body using the example in clientcommands.cpp




#else //Don't use DEFINE_CVAR, DEFINE_SERVER_COMMAND, or DEFINE_CLIENT_COMMAND beyond this point, or things break

#ifndef _PLUGIN_H_
#define _PLUGIN_H_

//=================================================================
//Plugin Information
//=================================================================
#define MY_DATE			__TIMESTAMP__
#define MY_NAME			"PhaseSelect_MM"
#define MY_AUTHOR		"James Cash (orig: voogru)"
#define MY_EMAIL		"theqizmo@pacbell.net (orig: voogru@voogru.com)"
#define MY_URL			"http://www.modns.org (orig: http://www.voogru.com)"
#define MY_LOGTAG		"PHASESELECT_MM"
#define MY_VERSION		"1.3"
#define MY_LOADABLE		PT_CHANGELEVEL
#define MY_UNLOADABLE	PT_CHANGELEVEL

#define MY_VERS_DWORD	1,3,0,0
#define MY_COMMENTS		"Thanks to: prsearle, [WHO]Them, and voogru."
#define MY_DESC			"PhaseSelect MetaMod plugin"
#define MY_FILENAME		MY_LOGTAG ".DLL"
#define MY_INTERNAL		MY_LOGTAG
#define MY_COPYRIGHT	"GNU Public License"

// Event for Phasing
#define	IP_EVENT		39

// The menu defaults
#define C_MENU_CHAN		28
#define C_MENU_X_COORD	0.6f
#define C_MENU_Y_COORD	0.35f

// Prints debug info
#define PG_DEBUG_MODE	0

// Other
#define ABSOLUTE_VALUE_EASY(x) (((x) < 0) ? (-(x)) : (x)) //very useful for gpGlobals->time comparisons
#define MAX_MENU_CHUNK 176

#ifndef VERS_PLUGIN_H	//so that we don't confuse the resource compiler
#include <extdll.h>		//always

#include <string.h>		//to prevent linux errors while processing meta_api.h
#include <meta_api.h>

#include "utilfunctions.h"

#define CREATE_CVAR_EXTERNS
	#include "complexmacros.h"	
#undef CREATE_CVAR_EXTERNS

typedef struct locationinfo_s
{
	char			Name[256];
	Vector			Origin[2];
} locationinfo_t;

typedef struct linkedentity_s
{
	int				iEntityIndex;
	Vector			vecOrigin;
	linkedentity_s	*pNext;
	linkedentity_s	*pPrev;
} phasegate_t, movement_t;

typedef struct playerdata_s
{
	int				iInMenu;
	float			fEntAct;
	float			flMenuDelay;
	linkedentity_s	*pLastEntUsed;
} playerdata_t;

//=================================================================
//an EXTREMELY useful setup for dynamically sized data
//=================================================================
struct DynDataAutoCleanUp 
{ 
	//used for easy initialization constancy
	bool			bisInitialized;
	bool			bDidMsg;

	// Global Message Vars
	int				gmsgTextMsg;
	int				gmsgScreenFade;
	int				gmsgShowMenu;

	// Number of ents
	int				iNumPhases;
	int				iNumMCs;

	// Other
	int				NumLocations;
	int				iCombat;

	// Time var for next ent checks
	float			NextEntityCheck;

	// For Menus
	locationinfo_t	LocationInfo[128];

	// Player data
	playerdata_t	PlayerData[33];

	// Important list pointers
	phasegate_t		*pPhaseGateHead;
	phasegate_t		*pPhaseGateTail;

	movement_t		*pMovementHead;
	movement_t		*pMovementTail;

	//set default values
	DynDataAutoCleanUp( void ) 
	{
		bisInitialized = false;

		pPhaseGateHead = NULL;
		pPhaseGateTail = NULL;

		pMovementHead = NULL;
		pMovementTail = NULL;

		iNumPhases = iNumMCs = 0;
	}

	//the entire reason this class exists, auto-cleanup
	~DynDataAutoCleanUp( void ) 
	{	//guaranteed to be called when the plugin is being unloaded
		if (bisInitialized) 
		{
			//just in case
			bisInitialized = false; 

			DeleteAllNodes();
			DeleteAllNodesM();

			pPhaseGateHead = NULL;
			pPhaseGateTail = NULL;

			pMovementHead = NULL;
			pMovementTail = NULL;
		}
	}

	void Initialize( void ) 
	{
		iCombat = 0;
		NumLocations = 0;

		DeleteAllNodes();
		DeleteAllNodesM();

		memset(PlayerData, 0, sizeof(playerdata_t) * 33);

		//return if we're already initialized
		if (bisInitialized) 
			return; 

		iNumPhases = iNumMCs = 0; 

		pPhaseGateHead = NULL;
		pPhaseGateTail = NULL;

		pMovementHead = NULL;
		pMovementTail = NULL;

		//so we don't initialize more than once :P
		bisInitialized = true;

		//most of the time the dynamically sized data is based on players
		if (gpGlobals->maxClients == 0) 
			return; 
	}

	// Functions for both the movement and the phasegate linked list
	// If you want to go back and make this ugly, and half the size, feel free =p

	// Appends a node to the end of the list
	void AppendNode (phasegate_t *pNode)
	{
		if (pPhaseGateHead == NULL) 
		{
			pPhaseGateHead = pNode;
			pNode->pPrev = NULL;
		}
		else 
		{
			pPhaseGateTail->pNext = pNode;
			pNode->pPrev = pPhaseGateTail;
		}

		pPhaseGateTail = pNode;
		pNode->pNext = NULL;

		iNumPhases++;
	}

	// Appends a node to the end of the list
	void AppendNodeM (movement_t *pNode)
	{
		if (pMovementHead == NULL) 
		{
			pMovementHead = pNode;
			pNode->pPrev = NULL;
		}
		else 
		{
			pMovementTail->pNext = pNode;
			pNode->pPrev = pMovementTail;
		}

		pMovementTail = pNode;
		pNode->pNext = NULL;

		iNumMCs++;
	}

	// Inserts a node into the list after pAfter
	void InsertNode (phasegate_t *pNode, phasegate_t *pAfter)
	{
		pNode->pNext = pAfter->pNext;
		pNode->pPrev = pAfter;

		if (pAfter->pNext != NULL)
			pAfter->pNext->pPrev = pNode;
		else
			pPhaseGateTail = pNode;

		pAfter->pNext = pNode;
		iNumPhases++;
	}

	// Inserts a node into the list after pAfter
	void InsertNodeM (movement_t *pNode, movement_t *pAfter)
	{
		pNode->pNext = pAfter->pNext;
		pNode->pPrev = pAfter;

		if (pAfter->pNext != NULL)
			pAfter->pNext->pPrev = pNode;
		else
			pMovementTail = pNode;

		pAfter->pNext = pNode;
		iNumMCs++;
	}

	// Removes the specified node from the list
	void RemoveNode (phasegate_t *pNode)
	{
		if (pNode->pPrev == NULL)
			pPhaseGateHead = pNode->pNext;
		else
			pNode->pPrev->pNext = pNode->pNext;

		if (pNode->pNext == NULL)
			pPhaseGateTail = pNode->pPrev;
		else
			pNode->pNext->pPrev = pNode->pPrev;

		delete pNode;

		iNumPhases--;
	}

	// Removes the specified node from the list
	void RemoveNodeM (movement_t *pNode)
	{
		if (pNode->pPrev == NULL)
			pMovementHead = pNode->pNext;
		else
			pNode->pPrev->pNext = pNode->pNext;

		if (pNode->pNext == NULL)
			pMovementTail = pNode->pPrev;
		else
			pNode->pNext->pPrev = pNode->pPrev;

		delete pNode;

		iNumMCs--;
	}

	// Deletes the entire list
	void DeleteAllNodes()
	{
		while (pPhaseGateHead != NULL)
			RemoveNode(pPhaseGateHead);

		iNumPhases = 0;
	}

	// Deletes the entire list
	void DeleteAllNodesM( void )
	{
		while (pMovementHead != NULL)
			RemoveNode(pMovementHead);

		iNumMCs = 0;
	}

	void CheckNodes( bool bIsMC )
	{
		linkedentity_s *pTemp = NULL;
		linkedentity_s *pPrev = NULL;

		if (bIsMC)
			pTemp = pMovementHead;
		else
			pTemp = pPhaseGateHead;

		// Delete any dangling nodes for whichever list we are
		// checking
		while (pTemp != NULL)
		{
			edict_t *pEdict = INDEXENT(pTemp->iEntityIndex);

			//  It's null/dead
			if (FNullEnt(pEdict))
			{
				pPrev = pTemp->pNext;
				RemoveNode(pTemp);
				pTemp = pPrev;
				continue;
			}
			//  It's in use mode
			else if (pEdict->v.iuser1 == 2)
			{
				// But its not built
				if (!UTIL_IsBuilt(pEdict))
				{
					pPrev = pTemp->pNext;
					RemoveNode(pTemp);
					pTemp = pPrev;
					continue;
				}
			}
			
			if (pTemp != NULL)
				pTemp = pTemp->pNext;
		}
	}

};

//make a global across all files of the above struct, actual declaration in plugin.cpp
extern DynDataAutoCleanUp	DynamicSizedData; 
extern hudtextparms_t		TextMenuSettings;

//=================================================================
//helper functions
//=================================================================
void plugin_init(void);
void plugin_quit(void);

//simplify the process of registering a user message without stepping on anyone else's code unless we have to
inline void SAFE_USER_MSG(int &gmsgvar, const char *szMsgName, const int &iSize) 
{
	if (gmsgvar == 0) 
	{
		//attempt to get the msg from the game that's running
		gmsgvar = GET_USER_MSG_ID(&Plugin_info, szMsgName, NULL);
		
		//and if that fails, register it ourselves
		if (gmsgvar == 0) 
			gmsgvar = REG_USER_MSG(szMsgName, iSize);
	}
}


#endif // #ifndef VERS_PLUGIN_H
#endif // #ifndef _PLUGIN_H_
#endif // #ifdef SETTING_UP_CVARS_AND_COMMANDS
