#include "plugin.h"
#include <sdk_util.h>
#include <cbase.h>

#include <gamerules.h>
#include <saverestore.h>
#include <time.h>
#include <shake.h>
#include <decals.h>
#include <player.h>

// Extra
DLL_GLOBAL const Vector g_vecZero = Vector(0,0,0);

//=========================================================
// UTIL_LogPrintf - Prints a logged message to console.
// Preceded by LOG: ( timestamp ) < message >
//=========================================================
void UTIL_LogPrintf( char *fmt, ... )
{
	va_list			argptr;
	static char		string[1024];
	
	va_start ( argptr, fmt );
	vsprintf ( string, fmt, argptr );
	va_end   ( argptr );

	// Print to server console
	ALERT( at_logged, "%s", string );
}

unsigned short FixedUnsigned16( float value, float scale )
{
	int output;

	output = (int)(value * scale);
	if ( output < 0 )
		output = 0;
	if ( output > 0xFFFF )
		output = 0xFFFF;

	return (unsigned short)output;
}

short FixedSigned16( float value, float scale )
{
	int output;

	output = (int)(value * scale);

	if ( output > 32767 )
		output = 32767;

	if ( output < -32768 )
		output = -32768;

	return (short)output;
}

// returns a CBaseEntity pointer to a player by index.  Only returns if the player is spawned and connected
// otherwise returns NULL
// Index is 1 based
CBaseEntity	*UTIL_PlayerByIndex( int playerIndex )
{
	CBaseEntity *pPlayer = NULL;

	if ( playerIndex > 0 && playerIndex <= gpGlobals->maxClients )
	{
		edict_t *pPlayerEdict = INDEXENT( playerIndex );
		if ( pPlayerEdict && !pPlayerEdict->free )
		{
			pPlayer = CBaseEntity::Instance( pPlayerEdict );	
		}
	}
	
	return pPlayer;
}

CBaseEntity *UTIL_FindEntityByString( CBaseEntity *pStartEntity, const char *szKeyword, const char *szValue )
{
	edict_t	*pentEntity;

	if (pStartEntity)
		pentEntity = pStartEntity->edict();
	else
		pentEntity = NULL;

	pentEntity = FIND_ENTITY_BY_STRING( pentEntity, szKeyword, szValue );

	if (!FNullEnt(pentEntity))
		return CBaseEntity::Instance(pentEntity);
	return NULL;
}

CBaseEntity *UTIL_FindEntityByClassname( CBaseEntity *pStartEntity, const char *szName )
{
	return UTIL_FindEntityByString( pStartEntity, "classname", szName );
}

//Yes, this is n00b and icky. Shut up. I suck with file access.
void UTIL_LookupLocationTitle(char *pszTitle)
{
	//If it doesnt have a _ chances are its not a location title.
	if (!strstr(pszTitle, "_"))
		return;
	
	FILE * fp;
	fp = fopen ("ns/titles.txt", "r"); 
	
	int iFoundVar = 0;
	char szBuffer[1024]; 
	
	if (fp != NULL)    
	{          
		while (fgets (szBuffer, 1023, fp) != NULL)      
		{          
			if ((szBuffer[0] == '\n') || ((szBuffer[0] == '/') && (szBuffer[1] == '/')))  
				continue; 
			
			szBuffer[strlen(szBuffer)-1] = NULL;
			
			if (FStrEq(szBuffer, pszTitle) && strstr(szBuffer, "_")) { iFoundVar = 1; continue; }//We found the line,
			if (iFoundVar == 1) { iFoundVar = 2; continue; }
			if (iFoundVar == 2) { strncpy(pszTitle, szBuffer, 127); break; }
		}
		fclose (fp);
	}
}

bool UTIL_VecBetween( const Vector& point, const Vector& bottomLeft, const Vector& topRight )
{	
	if ( point.x >= bottomLeft.x 
		&& point.y >= bottomLeft.y 
		&& point.x <= topRight.x 
		&& point.y <= topRight.y)
	{
		return true;
	}
	return false;
}

char *UTIL_GetLocation( Vector &origin )
{
	int index = 0;

	for (int i = 0; i <= DynamicSizedData.NumLocations; i++)
	{
		if (UTIL_VecBetween(origin, DynamicSizedData.LocationInfo[i].Origin[0], DynamicSizedData.LocationInfo[i].Origin[1]))
		{
			index = i;
			break;
		}
	}

	if (!FStrEq(DynamicSizedData.LocationInfo[index].Name, ""))
		return (DynamicSizedData.LocationInfo[index].Name);
	else
		return "Unknown Location";
}

void UTIL_MovePlayer( edict_t *pPlayer, Vector vecOrigin )
{
	// Make phase events set 1
	PLAYBACK_EVENT_FULL( 0, pPlayer, IP_EVENT, 0, (float *)(&(pPlayer->v.origin)), (float *)&g_vecZero, 0.0, 0.0, 0, 0, 0, 0 );
	EMIT_SOUND(pPlayer, CHAN_ITEM, "misc/transport2.wav", 1, ATTN_NORM);

	// Send them to new phase origin
	pPlayer->v.velocity = Vector(0,0,0);
	pPlayer->v.origin = vecOrigin + Vector(0,0,80);

	// Push them off the phase gate.
	UTIL_MakeVectors(pPlayer->v.angles);
	pPlayer->v.velocity = pPlayer->v.velocity + gpGlobals->v_forward * 300;

	// Make phase events set 2
	PLAYBACK_EVENT_FULL( 0, pPlayer, IP_EVENT, 0, (float *)(&(pPlayer->v.origin)), (float *)&g_vecZero, 0.0, 0.0, 0, 0, 0, 0 );
	EMIT_SOUND(pPlayer, CHAN_ITEM, "misc/transport2.wav", 1, ATTN_NORM);

	// Set their screen to fade
	UTIL_ScreenFade( pPlayer, Vector(255, 255, 255), 100, 1, 0.25, 0);
}

void UTIL_ShowMenu( edict_t *pEdict, int slots, int displaytime, bool needmore, char *pText )
{
   if (DynamicSizedData.gmsgShowMenu == 0)
       SAFE_USER_MSG(DynamicSizedData.gmsgShowMenu, "ShowMenu", -1 );

   MESSAGE_BEGIN( MSG_ONE, DynamicSizedData.gmsgShowMenu, NULL, pEdict );

   WRITE_SHORT( slots );
   WRITE_CHAR( displaytime );
   WRITE_BYTE( needmore );
   WRITE_STRING( pText );

   MESSAGE_END();
}

int UTIL_CountBuildingsTeam( const char *building, int team )
{
	int iBuildingCount = 0;
	CBaseEntity *pBuilding = NULL;

	while (pBuilding = UTIL_FindEntityByClassname(pBuilding, building))
	{
		if (UTIL_IsBuilt(pBuilding->edict()) && pBuilding->pev->team == team)
			iBuildingCount++;
	}

	return iBuildingCount;
}

int UTIL_CountBuildings( const char *building )
{
	int iBuildingCount = 0;
	CBaseEntity *pBuilding = NULL;

	while (pBuilding = UTIL_FindEntityByClassname(pBuilding, building))
	{
		if (UTIL_IsBuilt(pBuilding->edict()))
			iBuildingCount++;
	}

	return iBuildingCount;
}

bool UTIL_IsBuilt( edict_t *pent )
{
	if (!(pent->v.iuser4 & (1<<2)))
	{
		return true;
	}
	else if (FStrEq(STRING(pent->v.classname),"team_hive") && pent->v.fuser1 >= 1000)
	{
		pent->v.iuser4 &= ~(1<<2);
		return true;
	}

	return false;
}

void UTIL_EnableMenu( CBaseEntity *pPlayer, char *szMessage, unsigned short uSlotFlags )
{
	if (DynamicSizedData.gmsgShowMenu == 0)
		SAFE_USER_MSG(DynamicSizedData.gmsgShowMenu, "ShowMenu", -1 );

	MESSAGE_BEGIN( MSG_ONE, DynamicSizedData.gmsgShowMenu, NULL, pPlayer->edict() );
		WRITE_SHORT( uSlotFlags );
		WRITE_CHAR( -1 );
		WRITE_BYTE( FALSE );
		WRITE_STRING( "\n" );
	MESSAGE_END();

	TextMenuSettings.holdTime = 1000.0f;
	UTIL_HudText( pPlayer, TextMenuSettings, szMessage );
}

void UTIL_DisableMenu( CBaseEntity *pPlayer )
{
	if (DynamicSizedData.gmsgShowMenu == 0)
		SAFE_USER_MSG(DynamicSizedData.gmsgShowMenu, "ShowMenu", -1 );

	MESSAGE_BEGIN( MSG_ONE, DynamicSizedData.gmsgShowMenu, NULL, pPlayer->edict() );
		WRITE_SHORT( 1 );
		WRITE_CHAR( 1 );
		WRITE_BYTE( FALSE );
		WRITE_STRING( "\n" );
	MESSAGE_END();

	TextMenuSettings.holdTime = 1.0f;
	UTIL_HudText( pPlayer, TextMenuSettings, "\n" );

	DynamicSizedData.PlayerData[ENTINDEX(pPlayer->edict())].fEntAct = 0;
}

void UTIL_ScreenFade( edict_t *pPlayer, Vector colour, int alpha, float duration, float holdtime, int flags )
{
	if (DynamicSizedData.gmsgScreenFade == 0)
		SAFE_USER_MSG(DynamicSizedData.gmsgScreenFade, "ScreenFade", 12);

	MESSAGE_BEGIN( MSG_ONE, DynamicSizedData.gmsgScreenFade, NULL, pPlayer );		// use the magic #1 for "one client" HACKHACK: Fixed fade message at 101.. if it starts giving errors, check here first!
	WRITE_SHORT( FixedUnsigned16( duration, 1<<12 ) );	// fade lasts this long
	WRITE_SHORT( FixedUnsigned16( holdtime, 1<<12 ) );	// fade lasts this long
	WRITE_SHORT( (unsigned short) flags );				// fade type (in / out)
	WRITE_BYTE( (unsigned char) colour.x );				// fade red
	WRITE_BYTE( (unsigned char) colour.y );				// fade green
	WRITE_BYTE( (unsigned char) colour.z );				// fade blue
	WRITE_BYTE( (unsigned char) alpha );				// fade blue
	MESSAGE_END();
}

void UTIL_HudText( CBaseEntity *pEntity, const hudtextparms_t &textparms, const char *pMessage )
{
	if ( !pEntity )
		return;

	MESSAGE_BEGIN( MSG_ONE, SVC_TEMPENTITY, NULL, pEntity->edict() );
		WRITE_BYTE( TE_TEXTMESSAGE );
		WRITE_BYTE( textparms.channel & 0xFF );

		WRITE_SHORT( FixedSigned16( textparms.x, 1<<13 ) );
		WRITE_SHORT( FixedSigned16( textparms.y, 1<<13 ) );
		WRITE_BYTE( textparms.effect );

		WRITE_BYTE( textparms.r1 );
		WRITE_BYTE( textparms.g1 );
		WRITE_BYTE( textparms.b1 );
		WRITE_BYTE( textparms.a1 );

		WRITE_BYTE( textparms.r2 );
		WRITE_BYTE( textparms.g2 );
		WRITE_BYTE( textparms.b2 );
		WRITE_BYTE( textparms.a2 );

		WRITE_SHORT( FixedUnsigned16( textparms.fadeinTime, 1<<8 ) );
		WRITE_SHORT( FixedUnsigned16( textparms.fadeoutTime, 1<<8 ) );
		WRITE_SHORT( FixedUnsigned16( textparms.holdTime, 1<<8 ) );

		WRITE_SHORT( FixedUnsigned16( textparms.fxTime, 1<<8 ) );
		
		if ( strlen( pMessage ) < 512 )
		{
			WRITE_STRING( pMessage );
		}
		else
		{
			char tmp[512];
			strncpy( tmp, pMessage, 511 );
			tmp[511] = 0;
			WRITE_STRING( tmp );
		}
	MESSAGE_END();
}

void UTIL_MakeVectors( const Vector &vecAngles )
{
	MAKE_VECTORS( vecAngles );
}

void EMIT_SOUND_DYN( edict_t *entity, int channel, const char *sample, float volume, float attenuation, int flags, int pitch )
{
	EMIT_SOUND_DYN2(entity, channel, sample, volume, attenuation, flags, pitch);
}

// "Overloading" CBaseEntity can cause problems, so, instead of creating
// 6 functions within CBaseEntity, we create 1, and then call one of these
// 2 util functions to do our work from within that function.
void UTIL_PhaseUse( CBaseEntity *pActivator, CBaseEntity *pThis )
{
	// Indices of the two entities
	int PlIndex = ENTINDEX(pActivator->edict());
	int PgIndex = ENTINDEX(pThis->edict());

	edict_t *pPpg = INDEXENT(PgIndex);
	edict_t *pPpl = INDEXENT(PlIndex);

#if DEBUG_MODE
	char sTemp[512] = "";
#endif

	char szMessage[512] = "";
	char szTemp[64] = "";

	if (FNullEnt(pPpg) || FNullEnt(pPpl))
		return;

	DynamicSizedData.CheckNodes( false );

	// Phasecount powers, activate!
	if (DynamicSizedData.iNumPhases < 2)
		return;

	// Aliens can't use phasegates, unless cvar is active
	if (pPpg->v.team != pPpl->v.team && (int)cvar_phaseselect_pg_use->value == 0)
		return;

	phasegate_t	*pPG = DynamicSizedData.pPhaseGateHead;
	phasegate_t	*pPrev = NULL;

	// Find current PG
	while (pPG != NULL)
	{
		if (pPG->iEntityIndex == PgIndex)
			break;
		else
			pPG = pPG->pNext;
	}

	// We didn't find the phasegate
	if (pPG == NULL)
		return;

	// Don't show menu if there are only two pgs, and force 
	// aliens to go to next phasegate in the list
	if (DynamicSizedData.iNumPhases == 2 || 
		pPpg->v.team != pPpl->v.team || 
		DynamicSizedData.PlayerData[PlIndex].iInMenu != 0 ||
		FStrEq( g_engfuncs.pfnInfoKeyValue( g_engfuncs.pfnGetInfoKeyBuffer( pActivator->edict() ), "phaseselect" ), "0"))
	{
		if (pPG->pNext == NULL)
			pPG = DynamicSizedData.pPhaseGateHead;
		else
			pPG = pPG->pNext;

		DynamicSizedData.PlayerData[PlIndex].iInMenu = 0;
		DynamicSizedData.PlayerData[PlIndex].fEntAct = 0;
		DynamicSizedData.PlayerData[PlIndex].flMenuDelay = gpGlobals->time + cvar_phaseselect_delay->value;
		UTIL_DisableMenu( pActivator );
		UTIL_MovePlayer( pActivator->edict(), pPG->vecOrigin );
		return;
	}	

	// So the marine can't phase around like a wild... marine.
	if (DynamicSizedData.PlayerData[PlIndex].flMenuDelay > gpGlobals->time)
		return;

	int iVar = (int)cvar_phaseselect_pg->value;

	switch (iVar)
	{
// Case 1
		case 1:
		{
	// Cycle to next phasegate
	if (pPG->pNext == NULL)
	{
		// We reached the end of the list
		if (DynamicSizedData.pPhaseGateHead != NULL)
		{
			pPG = DynamicSizedData.pPhaseGateHead;
		}
		// And the list is empty. Can you say, 'good game?'
		else
		{
			return;
		}
	}
	else
	{
		// Check to make sure next node is ok
		if (pPG->pNext != NULL)
		{
			pPG = pPG->pNext;
		}
		// It's not, uh oh..
		else
		{
			// Check the previous node
			if (pPG->pPrev != NULL)
			{
				pPG = pPG->pPrev;
			}
			// Somehow we managed to get past the node check. Kill me, quick!
			else
			{
				return;
			}
		}
	}

	// Assign previous node so we can tell from where we phased
	if (pPG->pPrev == NULL)
		pPrev = DynamicSizedData.pPhaseGateTail;
	else
		pPrev = pPG->pPrev;

#if DEBUG_MODE
	sprintf( sTemp, "Phasing to (%d, at x=%f y=%f z=%f) from (%d, at x=%f y=%f z=%f)\n", pPG->iEntityIndex, pPG->vecOrigin.x, pPG->vecOrigin.y, pPG->vecOrigin.z, pPrev->iEntityIndex, pPrev->vecOrigin.x, pPrev->vecOrigin.y, pPrev->vecOrigin.z );
	ALERT( at_aiconsole, sTemp );
#endif

	// Phase them
	UTIL_MovePlayer( pActivator->edict(), pPG->vecOrigin );

	// Reset variables
	DynamicSizedData.PlayerData[PlIndex].iInMenu = 0;
	DynamicSizedData.PlayerData[PlIndex].fEntAct = 0;
	DynamicSizedData.PlayerData[PlIndex].flMenuDelay = gpGlobals->time + cvar_phaseselect_delay->value;
		}
			break;
// Case 2
		case 2:
		{
	phasegate_t	*pPGNext = NULL;
	phasegate_t	*pPGPrev = NULL;

	// Assign nodes so we can we can phase
	if (pPG->pPrev == NULL)
		pPGPrev = DynamicSizedData.pPhaseGateTail;
	else
		pPGPrev = pPG->pPrev;

	if (pPG->pNext == NULL)
		pPGNext = DynamicSizedData.pPhaseGateHead;
	else
		pPGNext = pPG->pNext;

	// Create the menu for Next/Previous
	sprintf( szMessage, "Select Exit:\n\n" );
	sprintf( szTemp, "1. Next     - %s (%.fm)\n", UTIL_GetLocation(pPGNext->vecOrigin), UTIL_GetDistance(pPpg->v.origin, pPGNext->vecOrigin) );
	strcat( szMessage, szTemp );
	sprintf( szTemp, "2. Previous - %s (%.fm)\n\n", UTIL_GetLocation(pPGPrev->vecOrigin), UTIL_GetDistance(pPpg->v.origin, pPGNext->vecOrigin) );
	strcat( szMessage, szTemp );
	strcat( szMessage, "9. Cancel." );

	DynamicSizedData.PlayerData[PlIndex].iInMenu = 1;
	DynamicSizedData.PlayerData[PlIndex].pLastEntUsed = pPG;

	// Them's uber menu
	UTIL_EnableMenu( pActivator, szMessage, (1<<0) | (1<<1) | (1<<8) );

	DynamicSizedData.PlayerData[PlIndex].flMenuDelay = gpGlobals->time + cvar_phaseselect_delay->value;
	}
			break;
// Case 3
		case 3:
		{
	phasegate_t	*pMe = pPG;

	sprintf( szMessage, "Select Exit:\n\n" );

	int iThisNum = 1;

	pPG = DynamicSizedData.pPhaseGateHead;

	// Create the text
	while (pPG != NULL)
	{
		edict_t *pEdict = INDEXENT(pPG->iEntityIndex);

		// Only do 8 at a time.
		if (iThisNum == 8)
			break;

		// Skip same phasegate
		if (pPG->iEntityIndex != PgIndex)
		{
			sprintf( szTemp, "%d. %s (%.fm)\n", iThisNum, UTIL_GetLocation(pPG->vecOrigin), UTIL_GetDistance(pPpg->v.origin, pPG->vecOrigin) );
			strcat( szMessage, szTemp );
		}
		else
		{
			pPG = pPG->pNext;
			continue;
		}

		// Make sure we don't create an access violation
		if (pPG != NULL)
		{
			iThisNum++;
			pPG = pPG->pNext;
		}
	}

	unsigned short uSlots = (1<<0) | (1<<8);

	if (iThisNum == 8 && (((DynamicSizedData.iNumPhases - 1) % 7) != 0))
	{
		uSlots |= (1<<7);
		strcat( szMessage, "8. More\n" );
	}

	strcat( szMessage, "\n9. Exit\n" );

	DynamicSizedData.PlayerData[PlIndex].iInMenu = 1;
	DynamicSizedData.PlayerData[PlIndex].pLastEntUsed = pMe;
	DynamicSizedData.PlayerData[PlIndex].flMenuDelay = gpGlobals->time + cvar_phaseselect_delay->value;

	for (int i = 1; i < (iThisNum - 1); i++)
		uSlots |= (1<<i);

	// Them's uber menu
	UTIL_EnableMenu( pActivator, szMessage, uSlots );
		}
		default:
			break;
	}
}

// :|
void UTIL_MCUse( CBaseEntity *pActivator, CBaseEntity *pThis )
{

}

float UTIL_GetDistance( vec3_t OriginStart, vec3_t OriginEnd )
{
	return (float)((OriginStart - OriginEnd).Length() / 100);
}
